from petaldata.datasets.stripe.reports.customer_lists.abstract_mom_diff import AbstractMoMDiff

from petaldata.datasets.stripe.reports.customer_lists.new import New
from petaldata.datasets.stripe.reports.customer_lists.expansion_contraction import ExpansionContraction
from petaldata.datasets.stripe.reports.customer_lists.churned import Churned


# import numpy
# import pandas

# Configuration variables

api_key = None
api_base = "https://app.petaldata.app"
cache_dir = None

# API Datasets

from petaldata.datasets import *
from petaldata.datasets import abstract
from petaldata.datasets import hubspot
from petaldata.datasets import metabase
from petaldata.datasets import stripe
from petaldata.datasets.stripe.reports import *


from petaldata.storage import *


import pandas as pd
import datetime
from datetime import datetime
from datetime import date

import petaldata
from petaldata.datasets.stripe.reports.abstract_stripe_report import AbstractStripeReport
from petaldata.datasets.stripe.reports.mtd_revenue import MTDRevenue
from petaldata.datasets.stripe.reports import query_filters

class Summary(AbstractStripeReport):

  def __init__(self,invoices,tz='UTC',end_time=datetime.now().astimezone()):
    super().__init__(invoices,tz=tz,end_time=end_time)
    self.mtd_report = MTDRevenue(invoices,tz=tz,end_time=end_time,fullRange=False)

  def to_frame(self):
    pass
    

  def to_gsheet(self,creds,spreadsheet_title=None,worksheet_title="Summary"):
    print("Opening Google Sheet...title=",spreadsheet_title)

    # Must share sheet with "client_email" from JSON creds
    sh = self.gsheet_client(creds).open(spreadsheet_title)

    wks = self.find_or_create_wks(sh,worksheet_title) 

    df_mtd = self.mtd_report.to_frame()   

    print("\t...updating worksheet")
    # rev
    wks.cell('I6').value = df_mtd.amount_due_per_month.max()
    wks.cell('I7').value = df_mtd["amount_due_per_month (Previous Month)"].max()
    wks.cell('J6').value = df_mtd.amount_paid_per_month.max()
    wks.cell('J7').value = df_mtd["amount_paid_per_month (Previous Month)"].max()
    # customers
    wks.cell('I11').value = df_mtd.customers.max()
    wks.cell('I12').value = df_mtd["customers (Previous Month)"].max()
    # logging updated at
    wks.cell('I3').value = str(self.setup_time(datetime.now().astimezone(),tz=self.tz))
    print("\t...Done.")



import pandas as pd
import datetime
from datetime import datetime
from datetime import date

import petaldata
from petaldata.datasets.stripe.reports.customer_lists.abstract_mom_diff import AbstractMoMDiff
from petaldata.datasets.stripe.reports import query_filters

class ExpansionContraction(AbstractMoMDiff):
  """
  Generates a list of customers who were invoiced for different amounts in the current versus previous month.
  """

  def to_frame(self):
    """Returns a Dataframe of customers with revenue changes."""
    df = self.df_changes
    df.set_index("customer",inplace=True)
    df = self.cents_to_dollars(df,cols=self.currency_columns(df))
    df['change'] = df.amount_due_per_month_cur - df.amount_due_per_month_prev
    return df[['customer_email_prev','amount_due_per_month_prev',
    'amount_due_per_month_cur','change','created_prev','created_cur','subscription.plan.id_cur']]

  def to_gsheet(self,creds,spreadsheet_title=None,worksheet_title="Customers - Expansion+Contraction"):
    super().to_gsheet(creds,spreadsheet_title,worksheet_title)


import datetime
from datetime import datetime

from petaldata.datasets.stripe.reports.abstract_stripe_report import AbstractStripeReport
from petaldata.datasets.stripe.reports.adjusted_invoices import AdjustedInvoices

# Aggregrate Reports
from petaldata.datasets.stripe.reports.mrr_by_month import MRRByMonth
from petaldata.datasets.stripe.reports.revenue_by_plan_by_month import RevenueByPlanByMonth
from petaldata.datasets.stripe.reports.mtd_revenue import MTDRevenue
from petaldata.datasets.stripe.reports.summary import Summary

# Customer Lists
from petaldata.datasets.stripe.reports.customer_lists import *

def all(invoices,tz='UTC',end_time=datetime.now().astimezone()):
  reports = []
  res=map(lambda cls: cls(invoices,tz=tz,end_time=end_time), [MRRByMonth,RevenueByPlanByMonth,MTDRevenue,New,ExpansionContraction,Churned,Summary])
  return list(res)


import pandas as pd
import datetime
from datetime import datetime
from datetime import date

import petaldata
from petaldata.datasets.stripe.reports.customer_lists.abstract_mom_diff import AbstractMoMDiff
from petaldata.datasets.stripe.reports import query_filters

class New(AbstractMoMDiff):
  """
  Generates a list of new customers that were invoiced in the current month.
  """

  def to_frame(self):
    df = pd.merge(self.df_prev, self.df_cur, on='customer', how='right', suffixes=('_prev',''))
    df = self.cents_to_dollars(df,cols=self.currency_columns(df))
    df.set_index("customer",inplace=True)
    return df[df.amount_due_per_month_prev.isna()][['customer_email','amount_due_per_month',
    'amount_paid_per_month',
    'created','subscription.plan.id']]

  def to_gsheet(self,creds,spreadsheet_title=None,worksheet_title="Customers - New"):
    super().to_gsheet(creds,spreadsheet_title,worksheet_title)


import pandas as pd
import datetime
from datetime import datetime
from datetime import date

import petaldata
from petaldata.datasets.stripe.reports.abstract_stripe_report import AbstractStripeReport
from petaldata.datasets.stripe.reports import query_filters

class MTDRevenue(AbstractStripeReport):

    def __init__(self,invoices,tz='UTC',end_time=datetime.now().astimezone(), fullRange=True):
        super().__init__(invoices,tz=tz,end_time=end_time)
        self.fullRange = fullRange
        print("MTD - comparing ({} - {}) to previous month ({} - {})".format(self.cur_start,self.cur_end,self.prev_start,self.prev_end))

    @property
    def df_prev(self):
        return self.df[query_filters.by_day_query(self.df,self.prev_start,self.prev_end)]

    @property
    def df_cur(self):
        return self.df[query_filters.by_day_query(self.df,self.cur_start,self.cur_end)]

    def how_join(self,df1,df2):
        if len(df1.index) > len(df2.index):
            how = "left"
        else:
            how = "outer"

        return how

    def to_frame(self):
        prev_month_by_day = self.group_and_agg(self.df_prev)
        cur_month_by_day = self.group_and_agg(self.df_cur)

        df_mtd=cur_month_by_day.cumsum().reset_index().join(prev_month_by_day.cumsum().reset_index(), rsuffix=" (Previous Month)",
            how=self.how_join(cur_month_by_day,prev_month_by_day))
        df_mtd.index += 1 
        df_mtd.index.name = 'Day'

        # Hack to prevent Segfault stripping timezones from NaT values ... this changes the dtype to object
        df_mtd["created (Previous Month)"] = df_mtd["created (Previous Month)"].dt.date
        df_mtd.created = df_mtd.created.dt.date

        return df_mtd

    @property
    def cur_start(self):
        return self.end_timestamp.ceil(freq='D') - pd.offsets.MonthBegin(1)

    @property
    def cur_end(self):
        return self.end_timestamp

    @property
    def prev_start(self):
        return self.prev_end.ceil(freq='D') - pd.offsets.MonthBegin()

    @property
    def prev_end(self):
        if self.fullRange:
            e = self.cur_start
        else:
            e = self.cur_end - pd.DateOffset(months=1)

        return e

    @staticmethod
    def group_and_agg(df):
        grouped = df.groupby(pd.Grouper(key="created",freq="D")).agg(
            {'amount_paid_per_month': 'sum',
             'amount_due_per_month': 'sum', 
             'customer_email': pd.Series.nunique,
             'simulated': 'sum'}
        )
        grouped = MTDRevenue.cents_to_dollars(grouped, cols=[i for i in grouped.columns if 'amount' in i])    
        return grouped.rename(columns={"customer_email": "customers", "simulated": "ongoing_annual_subscriptions"})
    
    def to_gsheet(self,creds,spreadsheet_title=None,worksheet_title="MTD"):
        super().to_gsheet(creds,spreadsheet_title,worksheet_title)



import pandas as pd
import datetime
from datetime import datetime
from datetime import date

import petaldata
from petaldata.datasets.stripe.reports.abstract_stripe_report import AbstractStripeReport
from petaldata.datasets.stripe.reports import query_filters

class RevenueByPlanByMonth(AbstractStripeReport):

  def to_frame(self):
    by_plan = self.df[(self.df.billing_reason != "manual") & query_filters.time_range_query(self.df,self.df.created.min(),self.end_timestamp) & query_filters.billing_status_query(self.df)].groupby([pd.Grouper(key="created",freq="M"),"subscription.plan.id"]).agg(
        {
            "amount_due_per_month": "sum", 
            "amount_paid_per_month": "sum",
            "paid": 'sum',
            "created": 'count',
            "customer_email": pd.Series.nunique
        }
    )
    by_plan.rename(columns={"customer_email": 'customers'},inplace=True)
    by_plan = self.cents_to_dollars(by_plan, cols=[i for i in by_plan.columns if 'amount' in i])  

    return by_plan  

  def to_gsheet(self,creds,spreadsheet_title=None,worksheet_title="Monthly Revenue by Plan ID"):
    super().to_gsheet(creds,spreadsheet_title,worksheet_title)



import pandas as pd
import datetime
from datetime import datetime
from datetime import date

import petaldata
from petaldata.datasets.stripe.reports.abstract_stripe_report import AbstractStripeReport
from petaldata.datasets.stripe.reports import query_filters

class AbstractMoMDiff(AbstractStripeReport):
  def __init__(self,invoices,tz='UTC',end_time=datetime.now().astimezone()):
    super().__init__(invoices,tz=tz,end_time=end_time)

    self.prev_start,self.prev_end,self.cur_start,self.cur_end = self.create_time_ranges(self.end_timestamp)

  @property
  def df_prev(self):
    return self.df_for_time_range(self.prev_start,self.prev_end)

  @property
  def df_cur(self):
    return self.df_for_time_range(self.cur_start,self.cur_end)

  @staticmethod
  def currency_columns(df):
    return [i for i in df.columns if 'amount' in i]

  @property
  def df_prev_cur(self):
    return pd.merge(self.df_prev, self.df_cur, on='customer', how='inner', suffixes=('_prev','_cur'))

  @property
  def df_changes(self):
    return  self.df_prev_cur[self.df_prev_cur.amount_due_per_month_prev != self.df_prev_cur.amount_due_per_month_cur]
  
  def df_for_time_range(self,start_time,end_time):
    return self.df[(self.df.billing_reason != "manual") & query_filters.time_range_query(self.df,start_time,end_time) & query_filters.billing_status_query(self.df)]

  @staticmethod
  def create_time_ranges(cur_end):
      # cur_start ends up being in utc, not the tz 00:00 start time for the month.
      cur_start = cur_end.ceil(freq='D') - pd.offsets.MonthBegin(1)
      prev_end = cur_end - pd.DateOffset(months=1)
      prev_start = prev_end.ceil(freq='D') - pd.offsets.MonthBegin()

      print("Comparing ({} - {}) to previous month ({} - {})".format(cur_start,cur_end,prev_start,prev_end))

      return prev_start,prev_end,cur_start,cur_end


import pandas as pd
import datetime
from datetime import datetime
from datetime import date
import pygsheets

import petaldata
from petaldata.datasets.stripe.reports import query_filters

class AdjustedInvoices(object):
    def __init__(self,invoices,tz='UTC',end_time=datetime.now().astimezone()):
        self.df = invoices.df
        self.df = self.set_frame_tz(self.df,tz=tz)
        self.convert_annual_subs_to_monthly(self.df)
        self.df = self.add_simulated_annual_invoices(self.df)

    @staticmethod
    def set_frame_tz(df,tz = 'UTC'):
        for col in df.columns:
          if 'datetime64[ns' in str(df[col].dtype):
            df[col] = df[col].dt.tz_convert(tz)
        return df

    @staticmethod
    def convert_annual_subs_to_monthly(df):
        print("Converting annual subscriptions to monthly")
        per_month = lambda amount, interval : amount / 12.0 if (interval == 'year') else amount
        df['amount_due_per_month'] = df.apply(lambda row: per_month(row.amount_due,row["subscription.plan.interval"]), axis=1)
        df['amount_paid_per_month'] = df.apply(lambda row: per_month(row.amount_paid,row["subscription.plan.interval"]), axis=1)

    @staticmethod
    def add_simulated_annual_invoices(df):
        print("Adding simulated monthly invoices from annual invoices...")
        frames = []
        for index, row in df[query_filters.annual_query(df) & (df.amount_due > 0)].iterrows():
            for datetime in pd.date_range(start=row["subscription.current_period_start"], 
                                      end=row["subscription.current_period_end"]-pd.offsets.MonthBegin(2), 
                                      freq='MS'):
                date = datetime.date()
                simulated_row = row.copy()
                simulated_row.name = row.name+"_"+str(date)
                t=pd.Timestamp(datetime)
                simulated_row.date = t
                simulated_row.created = t
                simulated_row["simulated"] = 1
                frame = simulated_row.to_frame().transpose()
                frame.index.name = "id"
                frames.append(frame)

        print("\t...concating")
        df_combined = pd.concat([df]+frames,verify_integrity=True,sort=False)
        print("\t...assigning dtypes")
        return AdjustedInvoices.assign_dtypes(df,df_combined)

    @staticmethod
    def assign_dtypes(original_df,other_df):
        return other_df.astype(original_df.dtypes.to_dict())


import pandas as pd
import datetime
from datetime import datetime
from datetime import date
import pygsheets

import petaldata
from petaldata.datasets.stripe.reports import query_filters
from petaldata.datasets.stripe.reports.adjusted_invoices import AdjustedInvoices


class AbstractStripeReport(object):
  def __init__(self,invoices,tz='UTC',end_time=datetime.now().astimezone()):
    """
    Parameters
    ----------
    invoices : stripe.Invoice or stripe.reports.AdjustedInvoices
    tz : str
        The timezone all report data should be presented with.
    end_time: tz-aware datetime

    Returns
    -------
    AbstractStripeReport
    """
    self.df = invoices.df
    self.tz = tz
    if invoices.__class__.__name__ == 'Invoices':
      self.df = AdjustedInvoices(invoices,tz='UTC',end_time=datetime.now().astimezone()).df

    self._gsheet_client = None

    self.end_timestamp = self.setup_time(end_time,tz=tz)

  
  @staticmethod
  def strip_frame_tz(df):
    print("Stripping timezone from datetime columns.")
    df = df.copy()
    for col in df.columns:
          if 'datetime64' in str(df[col].dtype):
            df[col] = df[col].apply(lambda x:datetime.replace(x,tzinfo=None))
    return df

  @staticmethod
  def setup_time(dt,tz=None):
    t=pd.Timestamp(dt)
    t=t.tz_convert(tz)
    return t

  @staticmethod
  def cents_to_dollars(df,cols=None):
    print("Converting cents to dollars. Cols=",cols)
    df[cols]=df[cols]/100
    return df


  def gsheet_client(self,creds):
    if self._gsheet_client is None:
      self._gsheet_client = pygsheets.authorize(custom_credentials=creds.with_scopes(['https://www.googleapis.com/auth/spreadsheets', 'https://www.googleapis.com/auth/drive.metadata.readonly']))
      
    return self._gsheet_client;

  def find_or_create_wks(self,sh,worksheet_title):
    try:
      wks = sh.worksheet_by_title(worksheet_title)
      print("\t...opening existing worksheet. title=",worksheet_title)
    except pygsheets.exceptions.WorksheetNotFound:
      print("\t...creating new worksheet. title=",worksheet_title)
      wks = sh.add_worksheet(worksheet_title)

    return wks  


  def to_gsheet(self,creds,spreadsheet_title=None,worksheet_title=None):
    """
    Parameters
    ----------
    creds : google.oauth2.service_account.Credentials
        Google Authentication Credentials.
    spreadsheet_title : str
        The title of the Google spreadsheet to update. The spreadsheet must exist and the user associated with the creds must have read/write access to the sheet.
    worksheet_title: str
        The title of the worksheet to update (a worksheet is within a spreadsheet). The worksheet will be created if it doesn't exist.

    Returns
    -------
    None
    """
    frame = self.to_frame()

    print("Opening Google Sheet...title=",spreadsheet_title)

    # Must share sheet with "client_email" from JSON creds
    sh = self.gsheet_client(creds).open(spreadsheet_title)
    wks = self.find_or_create_wks(sh,worksheet_title)
    print("\t...updating worksheet")
    wks.clear()
    wks.set_dataframe(self.strip_frame_tz(frame),(1,1), copy_index=True, nan="")
    wks.cell('A1').value = frame.index.name
    print("\t...Done.")





import pandas as pd
import requests
import datetime
import os
from datetime import datetime
from datetime import date
import json

import petaldata
from petaldata import util
from petaldata.storage import *

class Dataset(object):
  CSV_FILE_PREFIX = "dataset"

  def __init__(self,base_pickle_filename=None):
    """
    Initializes the Dataset, attempting to load a previously saved dataframe via `base_pickle_filename`.

    Parameters
    ----------
    base_pickle_filename : str
      The name of the Pickle file the Dataset should be loaded from and saved to. If the file doesn't exit, it will be created when
      the Dataset is saved. If not provided, `default_base_pickle_filename()` is used.
    """
    self.csv_filename = None
    self.df = None
    self.__metadata = None
    if base_pickle_filename == None: base_pickle_filename = self.default_base_pickle_filename()
    self.local = Local(base_pickle_filename)
    self.s3 = S3.if_enabled(base_pickle_filename)
    self._load_from_cache()

  def download(self,created_gt=None,_offset=None, limit=None):
    """
    Downloads from PetalData and initializes a Pandas dataframe, accessible at `df`, with the downloaded data.

    The dataframe is not saved. Call `save()` following the download to persist the dataframe as a Pickle file.
    """
    self._download_csv(created_gt,_offset,limit)
    self._load_from_download()

    if self.df is None:
      print("\tUnable to load dataframe.")
    return self.df

  def upsert(self, created_gt=None):
    """
    Inserts new rows and updates existing rows.

    Parameters
    ----------
    created_gt : datetime
      Only downloads rows created/updated after this value. If `None`, the value is set to the `updated` attribute. This
      makes the operation behave more like an `INSERT` of new rows.
    """
    print("Upserting...")
    if created_gt is None:
      print("\tSetting created_gt=",self.updated)
      created_gt = self.updated
    else:
      print("\tcreated >",created_gt)
    new = type(self)(base_pickle_filename="tmp.pkl")
    new.download(created_gt=created_gt)
    self._upsert(new)
    return self

  def delete(self):
    print("Deleting...")
    if self.local.enabled == True: self.local.delete()
    if self.s3: self.s3.delete()

    self.df = None
    self.__metadata = None
    return self

  def save(self):
    print("Saving to Pickle file...")
    if self.local.enabled == True: self.local.save(self)
    if self.s3: self.s3.save(self)

    return True
    
  @property
  def updated(self):
    """
    Returns the `datetime` of most recent created row.
    """
    return self.df.created.max()

  def request_params(self,created_gt,_offset):
    pass

  def default_base_pickle_filename(self):
    return self.CSV_FILE_PREFIX+".pkl"

  def _load_from_cache(self):
    print("Attempting to load saved pickle file...")
    if (self.local.enabled == True): self.df = self.local.read_pickle_dataframe() 
    if (self.s3 is not None): self.df = self.s3.read_pickle_dataframe() 

    if self.df is None:
      print("\tNo cached files exist.")
    else:
      print("\t...Done. Dataframe Shape:",self.df.shape)
      self._log_df_info()

  def _download_csv(self,created_gt=None,_offset=None, limit=None):
    first_chunk = True
    start_time = datetime.now()
    filename = self.local.csv_download_filename(self.CSV_FILE_PREFIX+"_",start_time)
    print("Starting download to",filename,"...")
    with requests.get(self.api_url+".csv", headers=self.request_headers, params=self.request_params(created_gt,limit,_offset), stream=True) as r:
        
        try:
          r.raise_for_status()
        except requests.exceptions.HTTPError as error:
          if error.response.status_code == 403:
            raise AuthError(json.loads(error.response.text)["error"])
          else:
            raise error

        with open(filename, 'wb') as f:
            for chunk in r.iter_content(chunk_size=None):
                # TODO - how to handle 500 errors in chunks?
                if chunk: # filter out keep-alive new chunks
                    if (first_chunk):
                        first_chunk = False
                        print("\t...will update progress every 25 MB of data transfer.")
                    f.write(chunk)
                    size_in_mb = Local.file_size_in_mb(filename)
                    # TODO - this isn't working ... instead, wonder about reporting number of rows in the file.
                    if ((size_in_mb > 1) & (len(chunk) > 1000) & (size_in_mb % 25 == 0) ):
                        print("\tDownloaded", size_in_mb, "MB...")

    size_in_mb = Local.file_size_in_mb(filename)
    time_delta = datetime.now() - start_time
    print("\t...Done.\n\tFile Size=",size_in_mb,"MB." " Total Time=", 
          round(time_delta.seconds/60.0,2), "minutes", "\n\tLocation:", filename)
    self.csv_filename = filename
    return self.csv_filename

  def _log_df_info(self):
    if (self.df is not None) & ('created' in self.df.columns) & (self.df.shape[0] > 0):
      print("\tTime Range:",self.df.created.min(), "-", self.df.created.max())

  def _load_from_download(self,filename=None):
    if filename is None:
      filename = self.csv_filename
    else:
      filename = self.local.dir + filename
    print("Loading {} MB CSV file...".format(Local.file_size_in_mb(filename)))
    dataframe = pd.read_csv(filename,parse_dates = self.metadata.get("convert_dates"))
    dataframe.set_index(self.metadata.get("index"),inplace=True)
    self.df = dataframe
    print("\t...Done. Dataframe Shape:",self.df.shape)
    self._log_df_info()
    return self.df

  def _upsert(self,other_resource):
    old_count = self.df.shape[0]
    # only concat if rows found, otherwise dtypes can change
    # https://stackoverflow.com/questions/33001585/pandas-dataframe-concat-update-upsert
    if other_resource.df.shape[0] > 0:
      print("\t...Inserting new rows")
      # print("created before concat:",self.df.created.dtype,self.df.created.head(5))
      # print("other created before concat:",other_resource.df.created.dtype,other_resource.df.created.head(5))
      df = pd.concat([self.df, other_resource.df[~other_resource.df.index.isin(self.df.index)]])
      # print("created after concat:",df.created.dtype,df.created.head(5))
      print("\t...Updating existing rows")
      df.update(other_resource.df)
      # timezone is stripped after update. add back.
      df = self.set_date_tz(df)
      # print("created after set_date_z:",df.created.dtype,df.created.head(5))
      self.df = df

    new_count = self.df.shape[0] - old_count
    if new_count > 0:
      print("...Added {} new rows. Now with {} total rows.".format(new_count, self.df.shape[0]))
      self._log_df_info()
    else:
      print("...No new rows.")
    return self

  @property
  def metadata(self):
    if self.__metadata is None:
      self.__metadata = self.get_metadata()

    return self.__metadata

  def get_metadata(self):
    r = requests.get(self.api_url+"/metadata.pandas",  headers=self.request_headers)
    r.raise_for_status()
    metadata = r.json()
    print("Loaded metadata w/keys=",list(metadata.keys()))
    print(metadata)
    return metadata

  @staticmethod
  def set_date_tz(dataframe):
    for col in dataframe.columns:
      # this is a tz-naive timestamp. w/a tz would look like "datetime64[ns, UTC]"
      if dataframe[col].dtype == 'datetime64[ns]':
        dataframe[col] = dataframe[col].dt.tz_localize('UTC')
        # Strips out tz info so don't have to worry about comparing tz-aware datetimes
        # dataframe[col] = dataframe[col].dt.tz_convert(None)
    return dataframe

  @property
  def api_url(self):
    return petaldata.api_base + self.RESOURCE_URL

class AuthError(Exception):
  pass





import os
from petaldata.storage.abstract_storage import AbstractStorage
from petaldata import util
import pandas as pd

class Local(AbstractStorage):
  """
  Local Storage abstracts local file access patterns.
  """
  enabled = True
  dir = None

  DEFAUL_DIRECTORY_NAME = "petaldata_cache"

  def __init__(self,base_pickle_filename):
    """
    Initializes local storage, creating `petaldata.Local.dir` if it doesn't exist. If `petaldata.Local.dir` isn't specified,
    it is set to `petaldata.Local.default_directory_path`.
    """
    if (self.dir == None): self.dir = self.default_directory_path
    self._create_dir()
    super().__init__(base_pickle_filename)

  @property
  def default_directory_path(self):
    return os.getcwd() + "/{}/".format(self.DEFAUL_DIRECTORY_NAME)

  def _create_dir(self):
    """
    Creates `petaldata.Local.dir` if it doesn't exist.
    """
    if not self.dir_exists():
      print("Creating local storage directory={}".format(self.dir))
      os.makedirs(self.dir)

  def available(self):
    res = self.dir_exists()
    if res == False:
      print("The local storage directory does not exist at", self.dir)
    return res

  def dir_exists(self):
    return os.path.isdir(self.dir)

  def read_pickle_dataframe(self):
    if not self.pickle_file_exists():
      print("\tLocal Pickle file does not exist at",self.pickle_filename)
      return None
    print("\tLoading {} MB Pickle file...".format(self.file_size_in_mb(self.pickle_filename)))
    print("\t...",self.pickle_filename)
    self.df = pd.read_pickle(self.pickle_filename)
    return self.df

  def save(self,resource):
    print("\tSaving local file...")
    resource.df.to_pickle(self.pickle_filename)
    print("\t...Done. Size (MB)=",self.file_size_in_mb(self.pickle_filename))
    return self.pickle_filename

  def delete(self):
    if self.pickle_file_exists():
      print("\tDeleting local pickle file.\n\tSize (MB)=",self.file_size_in_mb(self.pickle_filename),"\n\t",self.pickle_filename)
      os.remove(self.pickle_filename)
    else:
      print("\tNo pickle file exists at",self.pickle_filename)

  @property
  def pickle_filename(self):
    return self.dir + self.base_pickle_filename

  def pickle_file_exists(self):
    return os.path.isfile(self.pickle_filename)

  @staticmethod
  def file_size_in_mb(filename):
    return AbstractStorage.bytes_to_mb(os.path.getsize(filename))

  def csv_download_filename(self,csv_file_prefix,start_time):
    return self.dir + csv_file_prefix + start_time.strftime("%Y%m%d-%H%M%S") + ".csv"



import petaldata
from petaldata.datasets.abstract import Dataset

class Card(Dataset):
  RESOURCE_URL = "/metabase/card/{id}"
  CSV_FILE_PREFIX = "metabase_card"

  def __init__(self,id):
    self.id = id
    super().__init__(base_pickle_filename=self.CSV_FILE_PREFIX+"_"+str(self.id)+".pkl")

  @property
  def api_url(self):
    return petaldata.api_base + self.RESOURCE_URL.format(id=self.id)

  @property
  def request_headers(self):
    return {
      "Authorization": "Bearer %s" % (petaldata.api_key,),
      "METABASE_URL": petaldata.datasets.metabase.url,
      "METABASE_USERNAME": petaldata.datasets.metabase.username,
      "METABASE_PASSWORD": petaldata.datasets.metabase.password,
      "METABASE_TOKEN": petaldata.datasets.metabase.token
    }

  def request_params(self,created_gt,limit,_offset):
    params = {}
    if created_gt is not None:
       params['created_gt'] = calendar.timegm(created_gt.timetuple())
    if _offset is not None:
        params['_offset'] = _offset
    return params



import petaldata
from petaldata.datasets.abstract import Dataset
import calendar

class Invoices(Dataset):
  RESOURCE_URL = '/stripe/invoices'
  CSV_FILE_PREFIX = "stripe_invoices"

  @property
  def request_headers(self):
    return {
      "Authorization": "Bearer %s" % (petaldata.api_key,),
      'STRIPE_API_KEY': petaldata.datasets.stripe.api_key
    }

  def request_params(self,created_gt,limit,_offset):
    params = {}
    if created_gt is not None:
       params['created_gt'] = calendar.timegm(created_gt.timetuple())
    if limit is not None:
       params['limit'] = limit
    if _offset is not None:
        params['_offset'] = _offset
    return params
